/*
 * Copyright 2014 CyberVision, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kaaproject.kaa.demo.smarthousedemo.data.persistent;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

public class DeviceSqlLiteHelper extends SQLiteOpenHelper {

    public static final String TABLE_DEVICES = "devices";
    public static final String COLUMN_ID = "_id";
    public static final String COLUMN_ENDPOINT_KEY = "endpoint_key";
    public static final String COLUMN_DEVICE_TYPE = "device_type";
    public static final String COLUMN_DEVICE_NAME = "device_name";

    private static final String DATABASE_NAME = "devices.db";
    private static final int DATABASE_VERSION = 1;
 
    // The database creation sql statement.
    private static final String DATABASE_CREATE = "create table "
        + TABLE_DEVICES + "(" + 
            COLUMN_ID + " integer primary key autoincrement, " + 
            COLUMN_ENDPOINT_KEY + " text not null, " +
            COLUMN_DEVICE_TYPE + " text not null, " +
            COLUMN_DEVICE_NAME + " text not null);";
    
    public DeviceSqlLiteHelper(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }
    
    @Override
    public void onCreate(SQLiteDatabase database) {
      database.execSQL(DATABASE_CREATE);
    }
    
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
      Log.w(DeviceSqlLiteHelper.class.getName(),
          "Upgrading database from version " + oldVersion + " to "
              + newVersion + ", which will destroy all old data");
      db.execSQL("DROP TABLE IF EXISTS " + TABLE_DEVICES);
      onCreate(db);
    }
}
